#############################################################################
# 
# zzLocale - Zope Locale Support Product
# Copyright (C) 2000 Digital Garage Inc.
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
# 
#############################################################################

from App.special_dtml import HTMLFile
from zLOG import LOG, ERROR
import os, stat, sys, fnmatch
from string import split, join, rfind, find
from App.FindHomes import SOFTWARE_HOME
from App.FactoryDispatcher import FactoryDispatcher
import Products
from App.ApplicationManager import Fake
import OFS
import Acquisition
import types

try:
    import locale    # Python locale module
    server_locale=locale.setlocale(locale.LC_ALL)
except ImportError:
    server_locale=''

from zLOG import format_exception

def find_langs(locales_dir):
    """
    Find languages in dir
    """
    isdir=os.path.isdir
    avail_langs=[]
    if isdir(locales_dir):
        for f in os.listdir(locales_dir):
            if f != 'CVS' and isdir(os.path.join(locales_dir, f)):
                avail_langs.append(f)
    return avail_langs        

        
class LocaleHTMLFile(HTMLFile):
    def __init__(self, old_name, container, kw={}):
        isfile=os.path.isfile

        # this casing isn't so nice...
        if type(container) == types.DictType:
            dft_form=container[old_name]
            container['dft_'+old_name]=dft_form
            pr=container[old_name+'__roles__']
            container['dft_'+old_name+'__roles__']=pr
        else: # container is a class object
            dft_form=getattr(container, old_name)
            setattr(container, 'dft_%s' % (old_name), dft_form)
        self.setName(old_name)

	# Keep a reference to the default form, just in case...
	self.dftForm = dft_form

        # tag to prevent processing of the same HTMLFile multiple times during
	# the recursion
	dft_form.globals['zlang']='dft'                                   # This is used to tag the HTMLFile so it is not processed again

        dft_filename=dft_form.raw

	# Zope bug:
	#
	# dft_form.raw was wrong under Windows, like
	# 'C:\\zope\\lib\\python\\Products\\ZCatalog/addZCatalog.dtml'
	# Notice the last speparator? It's a slash.
	# This bug is fixed in latest Zope releases, but here is a
	# quick fix for compatibility with previous ones:
	dft_filename = join(split(dft_filename, '/'), os.sep)

	# Is this a Zope bug?
        pf=find(dft_filename, 'Products')
        if pf != -1:
            base_dir=dft_filename[:pf]+\
                      join(split(dft_filename[pf:], os.sep)[:2], os.sep)
        else:
            base_dir=SOFTWARE_HOME
            
        rel_filename=dft_filename[len(base_dir)+len(os.sep):]
        prefix, dft_id=os.path.split(rel_filename)
        if dft_id[-5:] == '.dtml': dft_id=dft_id[:-5]

        locales_dir=os.path.join(base_dir, 'locales')
        
        langs=find_langs(locales_dir)
        for lang in langs:
            dtml_filename=os.path.join(locales_dir, lang, rel_filename)
            if isfile(dtml_filename):
                lang_filename='%s_%s' % (lang, old_name)
#                lang_file=HTMLFile(dft_id, os.path.split(dtml_filename)[0])
                args=[]
		args.append(dft_id)
		args.append(os.path.split(dtml_filename)[0])
		kw['zlang']=lang                                          # This is used to tag the HTMLFile so it is not processed again
                lang_file=apply(HTMLFile,args,kw)
                if type(container) == types.DictType:
                    container[lang_filename]=lang_file
                    container[lang_filename+'__roles__']=pr
                else:
                    setattr(container, lang_filename, lang_file)
                
    def __call__(self, *args, **kw):
        id=self.id()
        obj=args[0]
        klass=obj.__class__
        if klass != FactoryDispatcher:
            container=klass
        else: container=obj
        try: manage_form=getattr(container, 'dft_%s' % id)
	# That's why we kept a reference. (Note: why not use just this one directly?)
	except: manage_form = self.dftForm

        try:
            mapping=args[2]
            request=mapping['REQUEST']
        except IndexError:
            request=None
            
        langs=buildLangList(request)
        for lang in langs:
            if lang == 'en':
                break
            form_attr='%s_%s' % (lang, id)
            if hasattr(container, form_attr):
                manage_form=getattr(container, form_attr)
                break

        if klass != FactoryDispatcher:
            return apply(manage_form, args, kw)
        else:
            return apply(manage_form, args[1:], kw)
            

def installProductInterfaces():
    # i don't think this is the coolest way to do it, but it'll do for now
    products={}
    instance_classes={}
    for mt in Products.meta_types:
        products[mt['product']]=None
    for mc in Products.meta_classes.values():
        instance_classes[mc]=None

    # install module-level constructor interfaces
    for product in products.keys():
        pm='Products.%s' % product
        pmodule=sys.modules[pm]
        m=pmodule._m
        for k, v in m.items():
            try:
                # Check for the "already done" tag
                if v.__class__ == HTMLFile  and not v.globals.has_key('zlang'):
                    product_dir=os.path.split(pmodule.__file__)[0]
                    locale_form=LocaleHTMLFile(k, m, v.globals)
                    m[k]=locale_form
            except AttributeError: pass        


def installLocaleInterfaces(klass):
    manage_forms={}
    globals={}
    for k, v in klass.__dict__.items():
        try:
            # Check for the "already done" tag
            if v.__class__ == HTMLFile and not v.globals.has_key('zlang'):
		if manage_forms.has_key(str(v.__name__)):
		    manage_forms[str(v.__name__)].append(k)
		else:
		    manage_forms[str(v.__name__)]=[k]
		    globals[str(v.__name__)]=v.globals
	except AttributeError:
	    pass
    for o, names in manage_forms.items():
	locale_form=LocaleHTMLFile(names[0], klass, globals[o])
	for name in names:
	    setattr(klass, name, locale_form) 


def buildLangList(REQUEST=None):
    """Build up a languages preference list based on client and server
    language settings

    Note: Is there a better way to do this than store the preferences
    in the REQUEST?  Once per request for this list seems a must.
    """
    langs=[]

    if REQUEST:
	# the "we've already figured out language prefs for this request"-case
	if REQUEST.has_key('zlangprefs'):
	    return REQUEST['zlangprefs']

	# look for cookie or form data-based user overrides first        
	if REQUEST.form.has_key('zlang'):
	    langs.append(REQUEST.form['zlang'])
	if REQUEST.cookies.has_key('zlang'):
	    langs.append(REQUEST.cookies['zlang'])
	
	# add browser setting-based language preferences    
	if REQUEST.has_key('HTTP_ACCEPT_LANGUAGE'):
	    for lang in split(REQUEST['HTTP_ACCEPT_LANGUAGE'], ','):
		langs.append(split(lang,';',1)[0])

    # add the server locale, if available, to the end of the list
    if server_locale not in ('C', ''):
	langs.append(server_locale)

    if REQUEST: REQUEST['zlangprefs']=langs     

    return langs

class LocaleManager(Fake, OFS.SimpleItem.Item, Acquisition.Implicit):
    """Locale management"""
    manage=manage_main=HTMLFile('langSelect', globals())
    id        ='LocaleManagement'
    name=title='Locale Management'
    meta_type ='Locale Management'
    icon='p_/LocaleManagement_icon'

    manage_options=(
        {'label':'Locale', 'action':'manage_main',
         'help':('OFSP','Locale-Management_Locale.dtml')},
        )

    def manage_setLocale(self, REQUEST, RESPONSE):
	"""Change the current locale."""

	if REQUEST.form.has_key('newzlang'):
	    newzlang=REQUEST.form['newzlang']
	    REQUEST.set('zlang',newzlang)
	    if newzlang=='':
#		RESPONSE.expireCookie('zlang')
# The RESPONSE.expireCookie seems buggy...
# So I have to do the following instead:
		RESPONSE.setCookie('zlang','',path='/')
	    else:
		RESPONSE.setCookie('zlang',newzlang,path='/')
	return RESPONSE.redirect(REQUEST['URL1']+'/manage_main')



def installAllInterfaces(module,depth=0):
    """ Try to install all locale interfaces in Zope by recursion """
    try:
	for k, v in module.__dict__.items():
	    try:
		if type(v) == types.ModuleType:
	            theoDepth = len(split(v.__name__,'.'))-2
		else:
		    theoDepth = len(split(v.__module__,'.'))-1
	    except AttributeError:
		theoDepth = -1
	    if theoDepth == depth:
	        try:
	            if v.__class__ != LocaleHTMLFile:
		        installLocaleInterfaces(v)
		        installAllInterfaces(v,depth+1)
	        except AttributeError:
	            try:
		        installLocaleInterfaces(v)
		    except AttributeError:
		        pass
	            installAllInterfaces(v,depth+1)
    except AttributeError:
	pass
